package com.kmalbz

import io.ktor.application.*
import io.ktor.response.*
import io.ktor.request.*
import io.ktor.routing.*
import io.ktor.http.*
import io.ktor.gson.*
import io.ktor.features.*
import io.ktor.client.*
import io.ktor.client.engine.apache.*
import io.ktor.auth.*
import kotlin.reflect.*
import java.util.*
import org.apache.http.HttpException
import com.kmalbz.database.DatabaseFactory
import io.ktor.util.KtorExperimentalAPI

fun main(args: Array<String>): Unit = io.ktor.server.netty.EngineMain.main(args)

@KtorExperimentalAPI
@Suppress("unused") // Referenced in application.conf
@kotlin.jvm.JvmOverloads
fun Application.module(testing: Boolean = false) {
    install(ContentNegotiation) {
        gson {
        }
    }

    val client = HttpClient(Apache) {
    }

    install(Authentication) {
    }

    DatabaseFactory.init()

    routing {
        get("/") {
            call.respondText("HELLO WORLD!", contentType = ContentType.Text.Plain)
        }

        get("/json/gson") {
            call.respond(mapOf("hello" to "world"))
        }

        install(StatusPages) {
            exception<AuthenticationException> { _ ->
                call.respond(HttpStatusCode.Unauthorized)
            }

            exception<AuthorizationException> { _ ->
                call.respond(HttpStatusCode.Forbidden)
            }

            exception<HttpException> { _ ->
                call.respond(HttpStatusCode.BadRequest)
            }
        }

        OutputServiceRDBServer().apply {
            registerOutput()
        }
    }
}

class AuthenticationException : RuntimeException()
class AuthorizationException : RuntimeException()

